
# q-expansion of newform 3600.2.f.c, downloaded from the LMFDB on 05 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3600
weight = 2
poly_data = [1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 2], 1]]

hecke_ring_character_values = [[3151, [1, 0]], [901, [1, 0]], [2801, [1, 0]], [577, [-1, 0]]]
aps_data = [[0, 0], [0, 0], [0, 0], [0, 0], [-4, 0], [0, -3], [0, 3], [-4, 0], [0, 0], [-2, 0], [8, 0], [0, -1], [6, 0], [0, 6], [0, 4], [0, 3], [-12, 0], [14, 0], [0, -2], [8, 0], [0, 3], [-8, 0], [0, 6], [10, 0], [0, 1], [-6, 0], [0, 0], [0, -2], [18, 0], [0, -3], [0, 4], [4, 0], [0, -1], [4, 0], [-10, 0], [-16, 0], [0, 3], [0, 2], [0, 8], [0, 7], [12, 0], [-10, 0], [0, 0], [0, 7], [0, 5], [0, 0], [-12, 0], [0, 4], [0, -14], [10, 0], [0, 1], [0, 0], [-14, 0], [28, 0], [0, -5], [0, 0], [-18, 0], [24, 0], [0, -1], [22, 0], [0, -2], [0, 11], [0, 14], [8, 0], [0, -5], [0, -7], [28, 0], [0, 1], [0, 6], [-14, 0], [0, 13], [24, 0], [0, 4], [0, 1], [4, 0], [0, -12], [6, 0], [0, 19], [14, 0], [-26, 0], [-36, 0], [-10, 0], [-16, 0], [0, 15], [16, 0], [0, 2], [-30, 0], [0, -11], [-14, 0], [0, 4], [0, -14], [32, 0], [0, -8], [-36, 0], [-20, 0], [0, 8], [-2, 0], [-42, 0], [0, -2], [30, 0], [0, -18], [0, 9], [0, -2], [-6, 0], [12, 0], [0, 1], [0, -2], [0, -3], [40, 0], [26, 0], [0, -20], [0, 1], [0, -1], [-28, 0], [-32, 0], [-18, 0], [0, 2], [0, -24], [0, 7], [-4, 0], [6, 0], [0, 7], [0, -19], [0, -14], [20, 0], [-30, 0], [10, 0], [0, 0], [0, 16], [0, 13], [-4, 0], [0, 8], [-8, 0], [0, -1], [-26, 0], [30, 0], [0, -21], [0, -2], [0, 1], [-6, 0], [28, 0], [10, 0], [0, 24], [0, -18], [2, 0], [-40, 0], [0, -7], [0, -1], [4, 0], [0, -4], [0, 11], [-2, 0], [0, 26], [0, 0], [0, -14], [0, 0], [-16, 0], [-30, 0], [0, 21], [50, 0], [0, -30], [0, -7], [0, 24], [12, 0], [0, -29], [0, -16], [-40, 0], [0, -9], [14, 0], [0, -21], [20, 0], [-2, 0], [-24, 0], [0, -13], [24, 0], [-22, 0], [-4, 0], [26, 0], [0, -8], [-46, 0], [0, -28], [20, 0], [0, -23], [0, -9], [0, 28], [54, 0], [0, 11], [0, 10], [22, 0], [32, 0], [0, 7], [0, 18], [-12, 0], [2, 0], [0, 18], [0, -7], [50, 0], [0, 5], [0, 11], [0, 24], [46, 0], [40, 0], [0, 23], [-66, 0], [-12, 0], [0, -7], [-8, 0], [0, 6], [-6, 0], [-52, 0], [0, 17], [-54, 0], [0, -8], [0, 14], [-24, 0], [-54, 0], [0, 20], [46, 0], [0, 24], [0, 15], [-10, 0], [16, 0], [-14, 0], [0, -28], [0, -14], [-54, 0], [0, -15], [48, 0], [0, -32], [12, 0], [0, 29], [44, 0], [40, 0], [-58, 0], [0, -2], [0, 12], [46, 0], [0, -5], [4, 0], [40, 0], [0, 14], [44, 0], [0, 16], [34, 0], [0, 37], [24, 0], [0, 28], [-12, 0], [52, 0], [0, -28], [0, 3], [30, 0], [0, 0], [-10, 0], [0, 31], [-36, 0], [-10, 0], [0, -22], [0, 37], [0, 5], [0, 20], [0, -6], [-22, 0], [0, -27], [0, 3], [28, 0], [78, 0], [6, 0], [0, 22], [0, 3], [-18, 0], [0, -2], [0, -5], [-56, 0], [0, 17], [0, -8], [0, 6], [-14, 0], [10, 0], [-12, 0], [0, -12], [16, 0], [0, 0], [38, 0], [0, -14], [-80, 0], [0, 39], [0, 21], [0, 0], [2, 0], [82, 0], [0, 26], [0, 9], [12, 0], [0, 29], [-66, 0], [56, 0], [0, 3], [-60, 0], [0, -2], [0, 19], [0, 9], [-24, 0]]
